/* Copyright (c) 2003-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

/* This module defines how Scintilla markers are used in Komodo.
 *  http://www.scintilla.org/ScintillaDoc.html#Markers
 *
 * Marker are the images in the editor gutter for things like bookmarks,
 * breakpoints. Also line background color for current line, current line in a
 * debugger session, etc.
 */
// Note: maybe all this stuff should move into scintilla.xml.

// Marker numbers.
// Most of the ISciMoz.marker*() methods take a marker number argument.
// The higher the marker number the higher the marker's z-order.
// Here are the marker numbers that Komodo uses.
const MARKNUM_STDERR = 12; // used in terminal view
const MARKNUM_STDOUT = 11; // used in terminal view
const MARKNUM_CURRENT_LINE_BACKGROUND = 10;
const MARKNUM_STDIN_PROMPT = 9; // used in terminal view
const MARKNUM_INTERACTIVE_PROMPT_MORE = 8; // used in interactive shell
const MARKNUM_INTERACTIVE_PROMPT = 7; // used in interactive shell
const MARKNUM_BOOKMARK = 6;
const MARKNUM_DEBUG_CURRENT_LINE = 5;
const MARKNUM_SPAWNPOINT_ENABLED = 4;
const MARKNUM_SPAWNPOINT_DISABLED = 3;
const MARKNUM_BREAKPOINT_ENABLED = 2;
const MARKNUM_BREAKPOINT_DISABLED = 1;
const MARKNUM_TRANSIENTMARK = 0; // used in buffer view

// Include all markers *except* MARKNUM_CURRENT_LINE_BACKGROUND.
// - Want the Nth bitfield for marker N to be set iff that marker should
//   up in the symbol margin and cleared if the line background should be
//   colored.
const MARKERS_MASK_SYMBOLS = 0x03FF;


//XXX Obselete, but here for debugging. I changed some of the marker number
//    and that might result in the wrong marker getting used in some Komodo
//    code. Have to make sure we correct all of those.
//var Editor_Markers = {
//    // markerOrdering: the lower the number, the higher the marker's z-order
//    clb: 5, // current line background color
//    bm: 4, // bookmark
//    cl: 3, // current line
//    wb: 2, // working (ie, enabled) breakpoint
//    fb: 1 // false breakpoint
//}

var _gPixmaps = {};
function markers_getPixmap(filename)
{
    if (typeof(_gPixmaps[filename]) == 'undefined') {
        _gPixmaps[filename] = open_readPixmap(filename);
    }
    return _gPixmaps[filename];
}

// Setup the standard Komodo markers in the given SciMoz instance and
// return an appropriate mask for ISciMoz.setMarginMaskN(<n>, <mask>).
function markers_setup(scimoz) {
    //XXX Was this: scimoz.markerDefine(MARKNUM_CURRENT_LINE_BACKGROUND, 32);
    //scimoz.markerDefine(MARKNUM_CURRENT_LINE_BACKGROUND,
    //                    scimoz.SC_MARK_BACKGROUND);
    //XXX The colors for this marker are changed depending on if we are
    //    debugging or not and what the user's Komodo style is.

    scimoz.markerDefine(MARKNUM_BOOKMARK, scimoz.SC_MARK_ARROWDOWN);
    scimoz.markerSetFore(MARKNUM_BOOKMARK, RGB(0x00, 0x00, 0x00)); // black
    scimoz.markerSetBack(MARKNUM_BOOKMARK, RGB(0x00, 0xFF, 0xFF)); // cyan

    scimoz.markerDefine(MARKNUM_DEBUG_CURRENT_LINE, scimoz.SC_MARK_SHORTARROW);
    scimoz.markerSetFore(MARKNUM_DEBUG_CURRENT_LINE, RGB(0x8f, 0x00, 0x00));
    scimoz.markerSetBack(MARKNUM_DEBUG_CURRENT_LINE, rgb.yellow);

    scimoz.markerDefinePixmap(MARKNUM_SPAWNPOINT_ENABLED,
                              markers_getPixmap("chrome://komodo/skin/images/spawnpoint.xpm"));
    scimoz.markerDefinePixmap(MARKNUM_SPAWNPOINT_DISABLED,
                              markers_getPixmap("chrome://komodo/skin/images/spawnpoint_disable.xpm"));
    scimoz.markerDefinePixmap(MARKNUM_BREAKPOINT_ENABLED,
                              markers_getPixmap("chrome://komodo/skin/images/breakpoint_enabled.xpm"));
    scimoz.markerDefinePixmap(MARKNUM_BREAKPOINT_DISABLED,
                              markers_getPixmap("chrome://komodo/skin/images/breakpoint_disabled.xpm"));
    scimoz.markerDefinePixmap(MARKNUM_INTERACTIVE_PROMPT,
                              markers_getPixmap("chrome://komodo/skin/images/prompt.xpm"));

    scimoz.markerDefine(MARKNUM_INTERACTIVE_PROMPT_MORE, scimoz.SC_MARK_DOTDOTDOT);
    scimoz.markerSetFore(MARKNUM_INTERACTIVE_PROMPT_MORE, rgb.black);

    scimoz.markerDefine(MARKNUM_STDIN_PROMPT, scimoz.SC_MARK_CHARACTER+'%'.charCodeAt(0));
    scimoz.markerSetFore(MARKNUM_STDIN_PROMPT, rgb.red);

    scimoz.markerDefine(MARKNUM_STDOUT, scimoz.SC_MARK_EMPTY);
    //scimoz.markerSetBack(MARKNUM_STDOUT, RGB(0xFA, 0xFA, 0xFF));
    scimoz.markerDefine(MARKNUM_STDERR, scimoz.SC_MARK_EMPTY);
    //scimoz.markerSetBack(MARKNUM_STDERR, RGB(0xFF, 0xFA, 0xFA));

    //XXX Have to see if this is necessary, i.e. if anything helpful is
    //    being done with this.
    //// Failed breakpoints (mo.fb not at a breakable position)
    //scimoz.markerDefine(1, scimoz.SC_MARK_CIRCLE);
    //scimoz.markerSetFore(1, RGB (0x8f, 0x00, 0x00));
    //scimoz.markerSetBack(1, rgb.green);

    scimoz.markerDefine(MARKNUM_TRANSIENTMARK, scimoz.SC_MARK_EMPTY);

    return MARKERS_MASK_SYMBOLS;
}

