/* Copyright (c) 2000-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

// start the extension
registeredExtensions[registeredExtensions.length] = new peFolder();

function peFolder() {
    this.name = 'peFolder';
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
peFolder.prototype.constructor = peFolder;

peFolder.prototype.init = function() {
}

peFolder.prototype.registerCommands = function() {
    extensionManager.registerCommand('cmd_delete', this);
    extensionManager.registerCommand('cmd_importFromFS', this);
    extensionManager.registerCommand("cmd_importFromPackage",this);
    extensionManager.registerCommand("cmd_renameFolderPart",this);
}

peFolder.prototype.registerMenus = function() {
    extensionManager.createMenuItem(Components.interfaces.koIPart_folder,'Import from File System...','cmd_importFromFS');
    extensionManager.createMenuItem(Components.interfaces.koIPart_folder,'Import Package...','cmd_importFromPackage');
    extensionManager.createMenuItem(Components.interfaces.koIPart_folder,'Refresh Status','cmd_refreshStatus');
    extensionManager.createMenuItem(Components.interfaces.koIPart_folder,'Rename...','cmd_renameFolderPart');
    var menupopup = document.getElementById('folder_context');
    extensionManager.addMenuItem(Components.interfaces.koIPart_folder, menupopup);
}

function peFolder_ensureFolderAdd(popup) {
    if (popup.childNodes.length > 0) return;
    // clone the folder context menu to create a new menu
    var menupopup = document.getElementById('folder_context_popup');
    for (var i =0; i < menupopup.childNodes.length; i++) {
        popup.appendChild(menupopup.childNodes[i].cloneNode(1));
    }
}

peFolder.prototype.registerEventHandlers = function() {
}

peFolder.prototype.supportsCommand = function(command, item) {
try {
    //dump('for ' + command + '\n');
    if (!gFocusedProjectView) return false;
    var items = gFocusedProjectView.getSelectedItems();
    var havemultiple = items.length > 1;
    //dump('for ' + command + ', items.length == ' + items.length + '\n');
    switch (command) {
    case 'cmd_delete':
        //HACK: Current behaviour for read-only toolbox's (typically the
        //      Shared Toolbox is to allow modifications and deal with the
        //      "can't save it" issues on save. Because adds and edits are
        //      allowed, it doesn't make sense to disallow delete. So we
        //      won't for now.
        //return gFocusedProjectView.manager.writeable();
        return true;
    case "cmd_importFromPackage":
    case "cmd_importFromFS":
        return items.length == 1 && !items[0].live;
    case "cmd_renameFolderPart":
        return items.length == 1 && !items[0].live && items[0].type == "folder";
    default:
        break;
    }
} catch (e) {
    log.exception(e);
}
    return false;
}

peFolder.prototype.isCommandEnabled = function(command, item) {
try {
    //dump('for ' + command + '\n');
    var items = null;

    if (gFocusedProjectView) {
        items = gFocusedProjectView.getSelectedItems();
        var havemultiple = items.length > 1;
    }
    switch (command) {
        case 'cmd_delete':
            // Delete is enabled for any part selection that isn't a project.
            if (view_getFocusedProjectView()) {
                if (!gFocusedProjectView.manager.writeable()) return false;
                item = gFocusedProjectView.getSelectedItem();
                if (!item) return false;
            }
            // leave cmd_delete enabled for any other element
            return true;
        case "cmd_importFromPackage":
        case "cmd_importFromFS":
            return items.length == 1 && !items[0].live;
        case "cmd_renameFolderPart":
            return items.length == 1 && !items[0].live && items[0].type == "folder";
        default:
            break;
    }
} catch (e) {
    log.exception(e);
}
    return false;
}

peFolder.prototype.doCommand = function(command) {
    var item = null;
    var firstchild, i;
    switch (command) {
    case 'cmd_addFilePart_Project':
        // XXX FIXME  nothing uses this???
        gProjectManager.viewMgr.focus();
        item = gProjectManager.getCurrentProject();
        // fall through
    case 'cmd_addFilePart':
        // XXX FIXME  nothing uses this???
        if (!item) item = gFocusedProjectView.getSelectedItem();
        peFolder_addFile(item);
        break;
    case 'cmd_addNewFile':
    case 'cmd_addNewFileFromTemplate':
        if (!item) item = gFocusedProjectView.getSelectedItem();
        peFolder_addNewFileFromTemplate(item);
        break;
    case 'cmd_addRemoteFilePart_Project':
        // XXX FIXME  nothing uses this???
        gProjectManager.viewMgr.focus();
        item = gProjectManager.getCurrentProject();
        // fall through
    case 'cmd_addRemoteFilePart':
        if (!item) item = gFocusedProjectView.getSelectedItem();
        peFolder_addRemoteFile(item);
        break;
    case 'cmd_addFolderPart_Project':
        // XXX FIXME  nothing uses this???
        gProjectManager.viewMgr.focus();
        item = gProjectManager.getCurrentProject();
        // fall through
    case 'cmd_addFolderPart':
        if (!item) item = gFocusedProjectView.getSelectedItem();
        var fitem = new Object();
        fitem.name = 'New Folder';
        fitem.value = null;
        if (gFocusedProjectView.manager.renameItem(fitem)) {
            this.addFolder(fitem.name, item);
        }
        break;
    case 'cmd_renameFolderPart':
        if (!item) item = gFocusedProjectView.getSelectedItem();
        if (item) {
            gFocusedProjectView.manager.renameItem(item)
            gFocusedProjectView.view.refresh(item);
        }
        break;
    case 'cmd_delete':
        // this cmd_delete handler only implements for our part viewers
        if (!view_getFocusedProjectView()) return;
        var items = gFocusedProjectView.getSelectedItems();
        if (items.length < 1) return;
        var havemultiple = (items.length > 1);
        var question = null;

        var removeText;
        if (gFocusedProjectView.manager.name == "projectManager") {
            removeText = "Remove from Project";
        } else {
            removeText = "Remove from Toolbox";
        }
        var haveLive = false;
        if (havemultiple) {
            question = "Do you want to remove the " + items.length +
                           " items you have selected?";
            for (i=0; i < items.length; i++) {
                var file = items[i].getFile();
                if (file && file.isLocal) {
                    haveLive = true;
                    break;
                }
            }
        } else {
            if (items[0].type == "project") break;
            question = "Do you want to remove the item you have selected?";
            var file = items[0].getFile();
            haveLive = file && file.isLocal;
        }
        var buttons;
        var text = null;
        if (haveLive) {
            buttons = ["Move to Trash", removeText, "Cancel"];
            text = "You may delete the files on disk, or simply remove the selected "+
                    "items from the project or toolbox.  If you choose only to remove "+
                    "the items and they are in a live folder, they will reappear later.";
        } else {
            buttons = [removeText, "Cancel"];
        }
        var action = dialog_customButtons(question, buttons, "Cancel", text,
                                        "Delete Selected Items",
                                        null, "warning-icon spaced")
        if (action == "Cancel")
            return;

        gFocusedProjectView.manager.removeItems(items, action == "Move to Trash");
        break;
    case "cmd_importFromFS":
        item = gFocusedProjectView.getSelectedItem();
        if (!item) return;
        part_ImportFromFS(item);
        gFocusedProjectView.view.refresh(item);
        gFocusedProjectView.view.selectPart(item);
        break;
    case "cmd_importFromPackage":
        item = gFocusedProjectView.getSelectedItem();
        if (!item) return;
        part_ImportFromPackage(item);
        gFocusedProjectView.view.refresh(item);
        gFocusedProjectView.view.selectPart(item);
        break;
    default:
        break;
    }
}

function peFolder_getDefaultDirectory(/*koIPart*/ parent)
{
    var defaultDirectory = null;
    var url = null;
    if (parent) {
        while (!url) {
            if (parent.hasAttribute('url'))
                url = parent.getStringAttribute('url')
            if (!parent.parent) break;
            parent = parent.parent;
        }
        if (url) {
            try {
                defaultDirectory = uriparse_URIToLocalPath(url);
                if (parent.type == 'project') {
                    var pathSvc = Components.classes["@activestate.com/koOsPath;1"]
                                            .getService(Components.interfaces.koIOsPath);
                    defaultDirectory = pathSvc.dirname(defaultDirectory);
                }
            } catch (ex) {
                // Throws exception when not right format, like a remote file.
                // Allows adding remote files to a remote folder in a project.
                // http://bugs.activestate.com/show_bug.cgi?id=66595
                defaultDirectory = url;
            }
        }
    }
    return defaultDirectory;
}

function peFolder_addNewFileFromTemplate(/*koIPart*/ parent)
{
    var defaultDirectory = peFolder_getDefaultDirectory(parent);
    return gViewMgr.newTemplate(defaultDirectory, parent.project);
}
    
function peFolder_addFileWithURL(url, /*koIPart*/ parent)
{
    if (typeof(parent)=='undefined' || !parent)
        parent = gFocusedProjectView.getSelectedItem();
    var part = parent.project.createPartFromType('file');
    part.setStringAttribute('url', url);
    part.setStringAttribute('name', uriparse_baseName(url));
    gFocusedProjectView.manager.addItem(part,parent);
}

function peFolder_add(type, partviewerId)
{
    try {
        var target, item, items, dirname, partviewer;
        if (!partviewerId) {
            target = gFocusedProjectView.getSelectedItem();
        } else {
            dump("adding "+type+" to viewer "+partviewerId+"\n");
            partviewer = document.getElementById(partviewerId);
            uilayout_ensureTabShown(partviewer.getAttribute('tabid'));
            partviewer.focus()
            target = partviewer.manager.getCurrentProject();
            gFocusedProjectView = partviewer;
        }

        // type can be:
        //   'files': add existing files
        switch (type) {
            case 'files':
                peFolder_addFile(target);
                break;
            case 'snippet':
                peSnippet_addSnippet(target);
                break;
            case 'command':
                peCommand_addCommand(target);
                break;
            case 'menu':
                peMenu_addMenu(target);
                break;
            case 'toolbar':
                peMenu_addToolbar(target);
                break;
            case 'URL':
                peURL_newURL(target);
                break;
            case 'macro':
                peMacro_addMacro(target);
                break;
            case 'remotefiles':
                peFolder_addRemoteFile(target);
                break;
            case 'template':
                peTemplate_addTemplate(target);
                break;
            case 'directoryshortcut':
                dirname = filepicker_getFolder();
                if (!dirname) return;
                peFile_addDirectoryShortcut(dirname, target);
                break;
            case 'newfile':
                peFolder_addNewFileFromTemplate(target);
                break;
            case 'newfolder':
                var name = dialog_prompt("Enter folder name:");
                if (!name) return;
                peFolder_addFolder(name, target);
                break;
            case 'livefolder':
                dirname = filepicker_getFolder();
                if (!dirname) return;
                peFolder_addLiveFolder(dirname, target);
                break;
            default:
                log.error("Unknown item type called to peFolder_add: " + type);
        }
        gFocusedProjectView.view.refresh(target);
    } catch (e) {
        log.error(e);
    }
}

function peFolder_addFile(parent_item)
{
    //XXX todo: Support other resources

    var defaultDir = null;
    if (parent_item && parent_item.url) {
        defaultDir = uriparse_URIToLocalPath(parent_item.url);
    }
    var files = filepicker_openFiles(defaultDir, // default dir
                                     null, // default filename
                                     "Add Files to Project"); // title
    if (files == null) {
        return false;
    } else {
        var url;
        for (var i = 0; i < files.length; ++i) {
            url = uriparse_localPathToURI(files[i]);
            peFolder_addFileWithURL(url, parent_item);
        }
        return true;
    }
}

function peFolder_addRemoteFile(item)
{
    var result = filepicker_remoteFileBrowser(); // in fileops.js
    if (result && result.filepaths.length > 0) {
        //dump("peFolder_addRemoteFile result.filepaths: " + result.filepaths+ "\n");
        for (var i = 0; i < result.filepaths.length; i++) {
            peFolder_addFileWithURL(result.filepaths[i], item);
        }
        return true;
    }
    return false;
}

function peFolder_addFolder(name, /*koIPart*/ parent)
{
    var folder = parent.project.createPartFromType('folder');
    folder.name = name;
    gFocusedProjectView.manager.addItem(folder,parent);
}

function peFolder_addLiveFolder(dirname, /*koIPart*/ parent)
{
    var folder = parent.project.createPartFromType('livefolder');
    folder.url = dirname;
    gFocusedProjectView.manager.addItem(folder,parent);
}

