/* Copyright (c) 2000-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

// start the extension
registeredExtensions[registeredExtensions.length] = new peSnippet();

var ANCHOR_MARKER = '!@#_anchor';
var CURRENTPOS_MARKER = '!@#_currentPos';

function peSnippet() {
    this.name = 'peSnippet';
    this.log = getLoggingMgr().getLogger('peSnippet');
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
peSnippet.prototype.constructor = peSnippet;

peSnippet.prototype.init = function() {
}

peSnippet.prototype.registerCommands = function() {
    extensionManager.registerCommand('cmd_insertSnippet', this);
    extensionManager.registerCommand('cmd_makeSnippetFromSelection', this);
}

peSnippet.prototype.registerEventHandlers = function() {
    extensionManager.addEventHandler(Components.interfaces.koIPart_snippet,'ondblclick',this);
}

peSnippet.prototype.registerMenus = function() {
    extensionManager.createMenuItem(Components.interfaces.koIPart_snippet,
                                    'Insert Snippet',
                                    'cmd_insertSnippet',
                                    null,
                                    null,
                                    true);
}

peSnippet.prototype.ondblclick = function(item,event) {
    if (item.type != 'snippet') return;
    Snippet_insert(item);
}

peSnippet.prototype.supportsCommand = function(command, part) {
    if (command == 'cmd_makeSnippetFromSelection') return true;
    var items = gFocusedProjectView.getSelectedItems();
    if (items.length > 1) return false;

    if (!gFocusedProjectView) return false;
    switch (command) {
    case 'cmd_insertSnippet':
        return true;
    default:
        break;
    }
    return false;
}


peSnippet.prototype.isCommandEnabled = function(command, part) {
    var items = null
    if (gFocusedProjectView) {
        items = gFocusedProjectView.getSelectedItems();
    }
    switch (command) {
    case 'cmd_makeSnippetFromSelection':
        var sel = '';
        if (gViewMgr.currentView &&
           gViewMgr.currentView.getAttribute('type') == 'editor') {
            sel = gViewMgr.currentView.selection;
        }
        return (sel != '');
    case 'cmd_insertSnippet':
        if (!gFocusedProjectView ||
            items.length > 1 ||
            !gViewMgr.currentView ||
            gViewMgr.currentView.getAttribute('type') != 'editor')
                return false;

        // XXX should just return true here, but the Extension manager isn't calling
        // isCommandEnabled, so we'll just hide the menu item for now.
        if (!part)
            part = gFocusedProjectView.getSelectedItem();
        return gViewMgr.currentView != null;
    default:
        break;
    }
    return false;
}

peSnippet.prototype.doCommand = function(command) {
    var item = null
    if (gFocusedProjectView) {
        item = gFocusedProjectView.getSelectedItem();
    }
    switch (command) {
    case 'cmd_insertSnippet':
        if (item) {
            Snippet_insert(item);
        }
        break;
    case 'cmd_makeSnippetFromSelection':
        var snippet = AddSnippetFromText(gViewMgr.currentView.selection);
        toolboxMgr.addItem(snippet,null);
    default:
        break;
    }
}

function snippet_editProperties (item)
{
    var obj = new Object();
    obj.item = item;
    obj.task = 'edit';
    openWindowMultipleInstance(
        "chrome://komodo/content/project/snippetProperties.xul",
        "komodo_snippetproperties",
        "chrome,close=yes,dependent=yes,resizable=yes", obj);
}

function peSnippet_addSnippet(/*koIPart*/ parent)
{
    var snippet = parent.project.createPartFromType('snippet');
    snippet.setStringAttribute('name', 'New Snippet');
    snippet.setStringAttribute('set_selection', 'false');
    snippet.setStringAttribute('indent_relative', 'false');
    snippet.value = '';
    var obj = new Object();
    obj.item = snippet;
    if (typeof(parent)=='undefined' || !parent)
        parent = gFocusedProjectView.getSelectedItem();
    obj.parentitem = parent;
    obj.gFocusedProjectView = gFocusedProjectView;
    obj.task = 'new';
    openWindowUniqueInstance(
        "chrome://komodo/content/project/snippetProperties.xul",
        "komodo_snippetProperties",
        "chrome,close=yes,dependent=yes,resizable=yes", obj);
}


function Snippet_insert (snippet) { // a part

    // Snippet insertion is surprisingly tricky to do.
    // Snippets are stored with escaped newlines.
    // Snippets also store the selection within the snippet using line/column markers.
    // These are converted to index positions, inserted into the text using special
    // markers, the text is run through the interpolation service, indented
    // according to the local context if appropriate; the markers are removed and
    // the selection is set (if required) after the insertion.
    var view = gViewMgr.currentView;
    if (!view || view.getAttribute('type') != 'editor') return;
    var scimoz = view.scimoz;
    var text, indent = '';
    scimoz.beginUndoAction();
    var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                        getService(Components.interfaces.koILastErrorService);
    try {
        try {
            view.setFocus()
            var eol = _getCurrentEOL();
            var setSelection = snippet.hasAttribute('set_selection') && snippet.getStringAttribute('set_selection') == 'true';
            var relativeIndent = snippet.hasAttribute('indent_relative') && snippet.getStringAttribute('indent_relative') == 'true';
            var viewData = Interpolate_getViewData(window);
            text = stringutils_unescapeWhitespace(snippet.value, eol);
            var encodingSvc = Components.classes["@activestate.com/koEncodingServices;1"].
                                getService(Components.interfaces.koIEncodingServices);
            var encoding = view.document.encoding.python_encoding_name;
            try {
                // XXX verify correctness of this when going between
                // two different non-utf-8 encodings
                if (encoding != 'utf-8')
                    text = encodingSvc.encode(text, encoding, 'strict');
            } catch (ex) {
                log.error(ex);
                alert("The snippet being inserted can't be converted to " +
                      "the current file's encoding.  You must" +
                      " change the file's encoding first. [" +
                      lastErrorSvc.getLastErrorMessage() + "]");
                return;
            }

            if (scimoz.selText.length == 0 && text.match(/%\(?[wW]/) != null) {
                // There is no selection but there is a '%w', '%W', '%(w', or '%(W' in the snippet
                // Special case: select the word
                if (Interpolate_isWordCharacter(scimoz.getWCharAt(scimoz.currentPos-1))) {  // There is part of a word to our left
                    scimoz.wordLeft();
                }
                // Using several wordPartRights instead of one wordRight because the latter is whitespace swallowing.
                while (Interpolate_isWordCharacter(scimoz.getWCharAt(scimoz.currentPos))) {  // There is part of a word to our right
                    scimoz.wordPartRightExtend();
                }
            }

            var anchor = 0;
            var currentPos = 0;
            var line, i;
            var before_selection, selection, after_selection;
            var lines = text.split(eol);

            // Do the interpolation of special codes
            text = text.replace('%%', '%', 'g');
            var istrings = Interpolate_interpolate(
                                window,
                                [], // codes are not bracketed
                                [text], // codes are bracketed
                                snippet.getStringAttribute("name"),
                                viewData);
            text             = istrings[0];
            //textForDisplay = istrings[1];

            if (relativeIndent) {
                // Do the indent
                var indentSize = scimoz.selectionStart - scimoz.positionFromLine(scimoz.lineFromPosition(scimoz.selectionStart))
                indent = '';
                var tabWidth;
                var useTabs = view.prefs.getBooleanPref("useTabs");
                for (i = 0; i < indentSize; i++) {
                    indent += ' ';
                }
                var tabequivalent = '';
                if (useTabs) {
                    tabWidth = view.prefs.getLongPref("tabWidth");
                    for (i = 0; i < tabWidth; i++) {
                        tabequivalent += ' ';
                    }
                }
                var parts = text.split(eol);
                var splits, newindent, rest;
                for (i = 0; i < parts.length; i++) {
                    if (i != 0 && parts[i].length != 0 && parts[i] != "<!@#_end>" && parts[i] != "<!@#_start>") {  // Do not indent empty lines.
                        parts[i] = indent + parts[i];
                        if (useTabs) {
                            newindent = '';
                            // This is ugly but I don't know a better way to do this in JS.
                            rest = parts[i].replace(/^\s*/, '');
                            newindent = parts[i].slice(0, parts[i].length-rest.length);
                            newindent = newindent.replace(tabequivalent, '\t', 'g');
                            parts[i] = newindent + rest;
                        }
                    }
                }
                text = parts.join(eol);
            }
            anchor = text.indexOf(ANCHOR_MARKER);
            currentPos = text.indexOf(CURRENTPOS_MARKER);
            if (anchor != -1 && currentPos != -1) {
                if (anchor < currentPos) {
                    anchor = text.indexOf(ANCHOR_MARKER);
                    text = text.replace(ANCHOR_MARKER, '');
                    currentPos = text.indexOf(CURRENTPOS_MARKER);
                    text = text.replace(CURRENTPOS_MARKER, '');
                } else {
                    currentPos = text.indexOf(CURRENTPOS_MARKER);
                    text = text.replace(CURRENTPOS_MARKER, '');
                    anchor = text.indexOf(ANCHOR_MARKER);
                    text = text.replace(ANCHOR_MARKER, '');
                }
            } else {
                anchor = 0;
                currentPos = 0;
            }
            scimoz.replaceSel("");
            var oldInsertionPoint = scimoz.currentPos;
            scimoz.insertText(oldInsertionPoint, text);
            if (setSelection) {
                scimoz.anchor = scimoz.positionAtChar(oldInsertionPoint,anchor);
                scimoz.currentPos = scimoz.positionAtChar(oldInsertionPoint,currentPos);
            } else {
                // selection will be after snippet
                scimoz.anchor = scimoz.positionAtChar(scimoz.anchor,text.length);
                scimoz.currentPos = scimoz.anchor;
            }
        } catch (ex) {
            var errno = lastErrorSvc.getLastErrorCode();
            if (errno == Components.results.NS_ERROR_ABORT) {
                // Command was cancelled.
            } else if (errno == Components.results.NS_ERROR_INVALID_ARG) {
                var errmsg = lastErrorSvc.getLastErrorMessage();
                alert("Error inserting snippet: " + errmsg);
            } else {
                log.error(ex);
                alert("There was an unexpected error: " + ex);
            }
        }
    } finally {
        macro_recordPartInvocation(snippet);
        scimoz.endUndoAction();
    }
}

function _AddSnippetFromText(snippettext, /*koIPart*/ parent) {
    // XX fix me
    // Important: The snippettext MUST be a unicode string
    var escapedtext = stringutils_escapeWhitespace(snippettext);
    escapedtext = escapedtext.replace('%', '%%', 'g');
    var snippet = parent.project.createPartFromType('snippet');
    snippet.type = 'snippet';
    snippet.setStringAttribute('name', snippetMakeDisplayName(snippettext));
    snippet.setStringAttribute('set_selection', 'true');
    snippet.setStringAttribute('indent_relative', 'false');
    escapedtext = ANCHOR_MARKER + escapedtext + CURRENTPOS_MARKER;
    snippet.value = escapedtext;
    return snippet;
}

function AddSnippetFromTextToToolbox(snippettext) {
    var snippet = _AddSnippetFromText(snippettext, toolboxMgr.toolbox);
    toolboxMgr.addItem(snippet, toolboxMgr.toolbox);
    return snippet;
}

function AddSnippetFromText(snippettext, /*koIPart*/ parent) {
    if (typeof(parent) == 'undefined' || !parent) {
        parent = gFocusedProjectView.manager.getCurrentProject();
    }
    var snippet = _AddSnippetFromText(snippettext, toolboxMgr.toolbox);
    gFocusedProjectView.manager.addItem(snippet, parent);
    return snippet;
}

/* Utility functions */

function _getCurrentEOL() {
    var view = gViewMgr.currentView;
    if (!view) {
        // we're not looking at anything -- return '\n' just to return something
        log.debug("toolboxManager: Asked for EOL when no view present...");
        return '\n'
    }
    var eol;
    if (view.document.new_line_endings == view.document.EOL_LF) {
        eol = '\n';
    } else if (view.document.new_line_endings == view.document.EOL_CR) {
        eol = '\r';
    } else if (view.document.new_line_endings == view.document.EOL_CRLF) {
        eol = '\r\n';
    } else {
        eol = '\n';
        log.error("toolboxManager: Unknown End of line")
    }
    return eol;
}

function snippetMakeDisplayName(text) {
    // Strip leading whitespace.
    text = text.replace( /^\s+/, "" );
    // Strip trailing whitespace.
    text = text.replace( /\s+$/, "" );
    if (text.length > 30) {
        text = text.substr(0, 20) + ' ... ' + text.substr(text.length - 10, text.length)
    }
    // Compress remaining whitespace.
    text = text.replace( /\s+/g, " " );
    return text
}



